shinyUI: Create a Shiny UI handler

View source: R/shinyui.R

shinyUIR Documentation

Create a Shiny UI handler

Description

[Superseded]

Historically this function was used in ui.R files to register a user interface with Shiny. It is no longer required; simply ensure that the last expression returned from ui.R is a user interface. This function is kept for backwards compatibility with older applications. It returns the value that is passed to it.

Usage

shinyUI(ui)

Arguments

ui

A user interface definition

Value

The user interface definition, without modifications or side effects.


shiny documentation built on June 21, 2026, 5:07 p.m.