[Superseded]

Historically this function was used in ui.R files to register a user interface with Shiny. It is no longer required; simply ensure that the last expression returned from ui.R is a user interface. This function is kept for backwards compatibility with older applications. It returns the value that is passed to it.

shinyUI(ui)

Arguments

ui

A user interface definition

Value

The user interface definition, without modifications or side effects.